<?php

class PerchUpgrade extends PerchAPI_Factory
{
    protected $table     = 'upgrade';
	protected $pk        = 'postID';


	public function find_importables($table_prefix='perch_')
	{
		if ($table_prefix=='') $table_prefix='perch_';

		$sql = 'SHOW TABLES LIKE '.$this->db->pdb(str_replace('_', '\_', $table_prefix).'%');
		$rows = $this->db->get_rows($sql);

		$found = array();

		if (PerchUtil::count($rows)) {
			foreach($rows as $row) {
				foreach($row as $table) {
					$short_name = str_replace($table_prefix, '', $table);
					$sql = 'SELECT COUNT(*) FROM '.$table;
					$item_count = $this->db->get_count($sql);

					$tmp = array(
						'short_name'=>$short_name,
						'table'=>$table,
						'prefix'=>$table_prefix,
						'item_count'=>$item_count
					);


					$found[] = $tmp;
				}
			}
		}

		return $found;
	}


	public function import_users($item)
	{
		$table = $item['table'];

		$sql = 'SELECT * FROM '.$table;
		$rows = $this->db->get_rows($sql);

		$out = array();

		if (PerchUtil::count($rows)) {
			$Users = new PerchUsers;

			foreach($rows as $row) {
				if (isset($row['userEmail'])) {
					$User = $Users->find_by_email($row['userEmail']);
					if (is_object($User)) {
						// user already exists
						$out[] = array('type'=>'failure',
										'messages'=>array(
												$row['userEmail'],
												'User already exists'
											));
					}else{
						// user doesn't exist, so import them
						
						$data = array();
						$data['userUsername'] 	 = $row['userUsername'];
						$data['userPassword'] 	 = $row['userPassword'];
						$data['userCreated'] 	 = $row['userCreated'];
						$data['userUpdated'] 	 = date('Y-m-d H:i:s');
						$data['userLastLogin'] 	 = $row['userLastLogin'];
						$data['userGivenName'] 	 = $row['userGivenName'];
						$data['userFamilyName']  = $row['userFamilyName'];
						$data['userEmail'] 		 = $row['userEmail'];
						$data['userEnabled'] 	 = $row['userEnabled'];
						$data['userHash'] 		 = $row['userHash'];
						$data['roleID'] 		 = ($row['userRole']=='Admin' ? 2 : 1);
						$data['userMasterAdmin'] = 0;

						$Users->create($data, $send_email=false);

						$out[] = array('type'=>'success',
										'messages'=>array(
												$row['userEmail'],
												'Successfully imported'
											));

					}
				}
			}
		}

		return $out;
	}


	public function import_settings($item)
	{
		$table = $item['table'];

		$sql = 'SELECT * FROM '.$table;
		$rows = $this->db->get_rows($sql);

		$out = array();

		if (PerchUtil::count($rows)) {
			$Settings = new PerchSettings;

			foreach($rows as $row) {
				if (isset($row['settingID'])) {
					$Setting = $Settings->get($row['settingID']);
					if (is_object($Setting) && $Setting->val()!='') {
						// Setting already exists
						$out[] = array('type'=>'failure',
										'messages'=>array(
												$row['settingID'],
												'Setting already exists'
											));
					}else{
						// Setting doesn't exist, so import it
						$data = array();
						$data['settingID'] 		= $row['settingID'];
						$data['userID'] 		= 0;
						$data['settingValue'] 	= $row['settingValue'];

						if ($row['settingID']=='logoPath' && $row['settingValue']=='/perch/assets/img/logo.png') {
							
							$out[] = array('type'=>'success',
										'messages'=>array(
												$row['settingID'],
												'Upgrading'
											));

						}else{

							$Settings->create($data);

							$out[] = array('type'=>'success',
										'messages'=>array(
												$row['settingID'],
												'Successfully imported'
											));

						}

						

					}
				}
			}
		}

		return $out;
	}


	public function import_content($item)
	{
		$table = $item['table'];

		$sql = 'SELECT DISTINCT contentPage FROM '.$table .' ORDER BY contentPage=\'*\' ASC, contentPage ASC';
		$rows = $this->db->get_rows($sql);

		$out = array();

		if (PerchUtil::count($rows)) {
			$Pages 	 = new PerchContent_Pages;
			$Regions = new PerchContent_Regions;
			$Items   = new PerchContent_Items;

			foreach($rows as $row) {
				if ($row['contentPage']!='*') {
					$Page = $Pages->find_or_create($row['contentPage']);	
				}else{
					$level0 = $Pages->get_by_parent(0);
					if (PerchUtil::count($level0)) {
						$Page = $level0[0];
					}else{
						continue;
					}
				}
				
				$out[] = array('type'=>'success',
										'messages'=>array(
												'<strong>'.PerchUtil::filename($row['contentPage'], false, false).'</strong>',
												$row['contentPage'],
												'Page imported'
											));

				$sql = 'SELECT * FROM '.$table.' WHERE contentPage='.$this->db->pdb($row['contentPage']).' ORDER BY contentOrder ASC';
				$regions = $this->db->get_rows($sql);

				if (PerchUtil::count($regions)) {
					foreach($regions as $region) {
						$data = array();
						$data['pageID']			  = $Page->id();
						$data['regionKey']		  = $region['contentKey'];
						$data['regionPage']		  = $region['contentPage'];
						$data['regionHTML']		  = $region['contentHTML'];
						$data['regionNew']		  = $region['contentNew'];
						$data['regionOrder']	  = $region['contentOrder'];
						$data['regionTemplate']	  = $region['contentTemplate'];
						$data['regionMultiple']	  = $region['contentMultiple'];
						$data['regionSearchable'] = $region['contentSearchable'];
						$data['regionRev']		  = 1;
						$data['regionLatestRev']  = 1;
						$data['regionOptions']	  = $region['contentOptions'];
						

						$opts = PerchUtil::json_safe_decode($region['contentOptions'], true);
						if (PerchUtil::count($opts)) {
							if (isset($opts['adminOnly']) && $opts['adminOnly']) {
								$data['regionEditRoles']  = '2';
							}
						}else{
							$data['regionEditRoles']  = '*';
						}

						$Region = $Regions->find_for_page_by_key($Page->id(), $region['contentKey']);

						if (is_object($Region)) {
							$out[] = array('type'=>'success',
										'messages'=>array(
												$region['contentKey'],
												'Region already imported, skipping'
											));
						}else{
							$Region = $Regions->create($data);
							$out[] = array('type'=>'success',
										'messages'=>array(
												$region['contentKey'],
												'Region successfully imported'
											));

							if (isset($region['contentAddToTop']) && $region['contentAddToTop']=='1') {
								$Region->set_option('addToTop', '1');
							}
						}

						// Items
						$contentItems = PerchUtil::json_safe_decode($region['contentJSON'], true);
						if (PerchUtil::count($contentItems)) {

							// Drop existing items for this region
							$Region->truncate(0);

							foreach($contentItems as $contentItem) {
								PerchUtil::debug($contentItem);
								$Item = $Region->add_new_item();

								$data = array();
								$data['itemJSON'] = PerchUtil::json_safe_encode($contentItem);
								$data['itemSearch'] = strip_tags($this->_implode_recursive(' ', $contentItem));

								$Item->update($data);							
							}

							$Region->publish();

								$count = PerchUtil::count($contentItems);
								$out[] = array('type'=>'success',
										'messages'=>array(
												$region['contentKey'],
												'Imported '.$count.' item'.($count==1?'':'s')
											));

						}

						


					}
				}

			}
		}

		return $out;
	}

	public function import_blog($item)
	{
		$table = $item['table'];

		$cat_table  		= str_replace('blog_posts', 'blog_categories', $table);
		$cat_to_post_table 	= str_replace('blog_posts', 'blog_posts_to_categories', $table);

		$sql = 'SELECT * FROM '.$table;
		$rows = $this->db->get_rows($sql);

		$out = array();

		if (PerchUtil::count($rows)) {
			$API = new PerchAPI(1.0, 'perch_blog');
			$Posts = new PerchBlog_Posts($API);
			$Categories = new PerchCategories_Categories();

			foreach($rows as $row) {
				if (isset($row['postSlug'])) {
					$Post = $Posts->get_one_by('postSlug', $row['postSlug']);
					if (is_object($Post)) {
						// post already exists
						$out[] = array('type'=>'failure',
										'messages'=>array(
												$row['postTitle'],
												'already imported'
											));
					}else{
						// post doesn't exist, so import it
						
						// Categories
						$sql = 'SELECT DISTINCT c.categoryTitle, c.categorySlug FROM '.$cat_table.' c, '.$cat_to_post_table.' p2c 
								WHERE p2c.categoryID=c.categoryID AND p2c.postID='.$this->db->pdb($row['postID']);
						$cats = $this->db->get_rows($sql);

						$cat_ids = array();

						if (PerchUtil::count($cats)) {
							foreach($cats as $cat) {
								$Category = $Categories->get_one_by('catSlug', $cat['categorySlug']);
								if (!is_object($Category)) {
									$Category = $Categories->create(array(
											'catTitle'=> $cat['categoryTitle'],
											'catSlug' => $cat['categorySlug']
										));
								}

								if (is_object($Category)) {
									$cat_ids[] = $Category->id();
								}
							}
						}

						if (count($cat_ids)) {
							$dynamicFields = PerchUtil::json_safe_decode($row['postDynamicFields'], true);
							$dynamicFields['categories'] = $cat_ids;
							$row['postDynamicFields'] = PerchUtil::json_safe_encode($dynamicFields);	
						}
						


						$data = array();
						$data['postTitle'] 			= $row['postTitle'];
						$data['postSlug'] 			= $row['postSlug'];
						$data['postDateTime'] 		= $row['postDateTime'];
						$data['postDescRaw'] 		= $row['postDescRaw'];
						$data['postDescHTML'] 		= $row['postDescHTML'];
						$data['postDynamicFields'] 	= $row['postDynamicFields'];
						$data['postTags'] 			= $row['postTags'];
						$data['postStatus'] 		= (isset($row['postStatus']) ? $row['postStatus'] : 'Published');
						

						$Post = $Posts->create($data);

						$Template   = $API->get('Template');
    					$Template->set('blog/'.$Post->postTemplate(), 'blog');

    					$Post->index($Template);


						$out[] = array('type'=>'success',
										'messages'=>array(
												$row['postTitle'],
												'Successfully imported'
											));

						

					}
				}
			}
		}

		return $out;
	}


	public function import_events($item)
	{
		$table = $item['table'];

		$cat_table  		= str_replace('events', 'events_categories', $table);
		$cat_to_post_table 	= str_replace('events', 'events_to_categories', $table);



		$sql = 'SELECT * FROM '.$table;
		$rows = $this->db->get_rows($sql);

		$out = array();

		if (PerchUtil::count($rows)) {
			$API = new PerchAPI(1.0, 'perch_events');
			$Events = new PerchEvents_Events($API);
			$Categories = new PerchEvents_Categories($API);

			foreach($rows as $row) {
				if (isset($row['eventSlug'])) {
					$Event = $Events->get_one_by('eventSlug', $row['eventSlug']);
					if (is_object($Event)) {
						// post already exists
						$out[] = array('type'=>'failure',
										'messages'=>array(
												$row['eventTitle'],
												'already imported'
											));
					}else{
						// post doesn't exist, so import it
						
						// Categories
						$sql = 'SELECT DISTINCT c.categoryTitle, c.categorySlug FROM '.$cat_table.' c, '.$cat_to_post_table.' p2c 
								WHERE p2c.categoryID=c.categoryID AND p2c.eventID='.$this->db->pdb($row['eventID']);
						$cats = $this->db->get_rows($sql);

						$cat_ids = array();

						if (PerchUtil::count($cats)) {
							foreach($cats as $cat) {
								$Category = $Categories->get_one_by('categorySlug', $cat['categorySlug']);
								if (!is_object($Category)) {
									$Category = $Categories->create($cat);
								}

								if (is_object($Category)) {
									$cat_ids[] = $Category->id();
								}
							}
						}

						$data = array();
						$data['eventTitle'] 		= $row['eventTitle'];
						$data['eventSlug'] 			= $row['eventSlug'];
						$data['eventDateTime'] 		= $row['eventDateTime'];
						$data['eventDescRaw'] 		= $row['eventDescRaw'];
						$data['eventDescHTML'] 		= $row['eventDescHTML'];
						$data['eventDynamicFields'] = @$row['eventDynamicFields'];
						$data['cat_ids']			= $cat_ids;

						$Events->create($data);

						$out[] = array('type'=>'success',
										'messages'=>array(
												$row['eventTitle'],
												'Successfully imported'
											));

						

					}
				}
			}
		}

		return $out;
	}

	public function import_gallery($item)
	{
		$table = $item['table'];

		$image_table  		= str_replace('gallery_albums', 'gallery_images', $table);
		$versions_table 	= str_replace('gallery_albums', 'gallery_image_versions', $table);


		$sql = 'SELECT * FROM '.$table;
		$rows = $this->db->get_rows($sql);

		$out = array();

		if (PerchUtil::count($rows)) {
			$API = new PerchAPI(1.0, 'perch_gallery');
			$Albums = new PerchGallery_Albums($API);
			$Images = new PerchGallery_Images($API);
			$Versions = new PerchGallery_ImageVersions($API);

			foreach($rows as $row) {
				if (isset($row['albumSlug'])) {
					$Album = $Albums->get_one_by('albumSlug', $row['albumSlug']);
					if (is_object($Album)) {
						// post already exists
						$out[] = array('type'=>'failure',
										'messages'=>array(
												$row['albumTitle'],
												'already imported'
											));
					}else{
						// album doesn't exist, so import it

						$data = array();
						$data['albumTitle'] 		= $row['albumTitle'];
						$data['albumSlug'] 			= $row['albumSlug'];
						$data['albumOrder'] 		= $row['albumOrder'];
						$data['albumDynamicFields'] = @$row['albumDynamicFields'];

						$Album = $Albums->create($data);

						$out[] = array('type'=>'success',
										'messages'=>array(
												$row['albumTitle'],
												'Album successfully imported'
											));

						// Images
						$sql = 'SELECT * FROM '.$image_table.' WHERE albumID='.$row['albumID'];
						$images = $this->db->get_rows($sql);

						if (PerchUtil::count($images)) {
							foreach($images as $image) {
								$data = array();

								$data['imageAlt'] 			= $image['imageAlt'];
								$data['imageSlug'] 			= $image['imageSlug'];
								$data['albumID'] 			= $Album->id();
								$data['imageOrder'] 		= $image['imageOrder'];
								$data['imageDynamicFields'] = $image['imageDynamicFields'];

								$Image = $Images->create($data);

								

								// Image versions
								
								$sql = 'SELECT * FROM '.$versions_table.' WHERE imageID='.$image['imageID'];
								$versions = $this->db->get_rows($sql);

								if (PerchUtil::count($versions)) {
									foreach($versions as $version) {
										$data = array();

										$data['versionPath'] 	= $version['versionPath'];
										$data['versionKey'] 	= $version['versionKey'];
										$data['versionWidth'] 	= $version['versionWidth'];
										$data['versionHeight'] 	= $version['versionHeight'];
										$data['imageID'] 		= $Image->id();

										$Versions->create($data);
									}
								} 

								$out[] = array('type'=>'success',
										'messages'=>array(
												$image['imageAlt'],
												'Image successfully imported'
											));

							}
						}

						

					}
				}
			}
		}

		return $out;
	}

	private function _implode_recursive($glue, $pieces)
	{
        $return = "";

        if(!is_array($glue)){
            $glue = array($glue);
        }
        
        $thisLevelGlue = array_shift($glue);

        if(!count($glue)) $glue = array($thisLevelGlue);
        
        if(!is_array($pieces)){
            return (string) $pieces;
        }
        
        foreach($pieces as $sub){
            $return .= $this->_implode_recursive($glue, $sub) . $thisLevelGlue;
        }

        if(count($pieces)) $return = substr($return, 0, strlen($return) -strlen($thisLevelGlue));

        return $return;
	}
}

?>